/* 
 * File:   main.c
 * Author: David Meyer
 *
 * Created on July 26, 2012
 *
 * USB Communication Board:
 * Select which mode of communication via pins B4 and B7
 * CS1: RB4
 * CS2: RB7
 *
 * 3 - Both High:   RS232
 * 2 - CS1 Low:     RS232 w/ Flow Control
 * 1 - CS2 Low:     SPI
 * 0 - Both Low:    I2C
 *
 *
 * Modified 7/26/2012 - David Meyer
 *
 */


// DEVCFG3:
#pragma config IOL1WAY  = OFF       // Peripheral Pin Select Configuration, allow mult reconfig
#pragma config PMDL1WAY = OFF       // Peripheral Module Disable Config, allow mult reconfig
// DEVCFG2:
#pragma config UPLLEN   = ON        // USB PLL Enable
#pragma config UPLLIDIV = DIV_2     // USB PLL Divider
// 8MHz / 2 * 24 / 2 = 48MHz
#pragma config FPLLODIV = DIV_2     // PLL Output Divider
#pragma config FPLLMUL  = MUL_20    // PLL Multiplier
#pragma config FPLLIDIV = DIV_2     // PLL Input Divider
// 8MHz / 2 * 20 / 2 = 40MHz

// DEVCFG1:
#pragma config FWDTEN   = OFF       // Watchdog Timer
#pragma config WDTPS    = PS1       // Watchdog Timer Postscale
#pragma config FCKSM    = CSDCMD    // Clock Switching & Fail Safe Clock Monitor
#pragma config FPBDIV   = DIV_1     // Peripheral Clock divisor
#pragma config OSCIOFNC = OFF       // CLKO Enable
#pragma config POSCMOD  = HS        // Primary Oscillator
#pragma config IESO     = OFF       // Internal/External Switch-over
#pragma config FSOSCEN  = OFF       // Secondary Oscillator Enable (KLO was off)
#pragma config FNOSC    = PRIPLL    // Oscillator Selection
// DEVCFG0:
#pragma config CP       = OFF       // Code Protect
#pragma config BWP      = ON        // Boot Flash Write Protect
#pragma config PWP      = OFF       // Program Flash Write Protect
#pragma config ICESEL   = ICS_PGx1  // ICE/ICD Comm Channel Select
#pragma config JTAGEN   = OFF       // JTAG Enable
#pragma config DEBUG    = OFF       // Background Debugger Enable

/** Includes *******************************************************/
#include "HardwareProfile.h"
#include "usb_rs232.h"
#include "usb_rs232_hfc.h"



/** PreCompiler Definitions ****************************************/
#define CS1         PORTBbits.RB4
#define CS2         PORTBbits.RB7




int main() {

    // enable multi-vector interrupts
    INTEnableSystemMultiVectoredInt();
    SYSTEMConfigPerformance(SYS_FREQ);

    // set all analog input pins to digital mode
    ANSELA = 0;
    ANSELB = 0;

    // Define B4 and B7 as digital inputs with pull-up resistors
    TRISBbits.TRISB4 = 1;
    TRISBbits.TRISB7 = 1;
    CNPUBbits.CNPUB4 = 1;
    CNPUBbits.CNPUB7 = 1;

    // Define B8 as a digital out for the LED
    TRISBbits.TRISB8 = 0;
    mLED = 0;


    USBOutHandle = NULL;
    USBInHandle = NULL;

    USBDeviceInit();
    USBDeviceAttach();



    mLED = 1;


    // Read in jumper locations
    int comm_select = 2*CS2+CS1;

    switch(comm_select)
    {
        case 0:         //I2C

            break;
        case 1:         //SPI

            break;
        case 2:         //RS232 w/ Flow Control
            RS232_HFC_setup();
            RS232_HFC_run();
            break;

        case 3:         //RS232
        default:
            RS232_setup();
            RS232_run();
            break;

    }

}
