/*
 * File:   usb_rs232.c
 * Author: David Meyer
 *
 * Created on July 26, 2012
 *
 * USB Communication Board:
 * Functions relating to translating between RS232 and USB
 *
 * Modified 7/30/2012 - David Meyer
 *
 */


/** Includes *******************************************************/
#include "HardwareProfile.h"

void RS232_setup(){

    RPA0Rbits.RPA0R = 0b0001;   // Pin2 - U1TX
    U1RXRbits.U1RXR = 0b0011;   // Pin24 - U1RX
    // turn on UART1 without an interrupt
    UARTConfigure(UART1, UART_ENABLE_PINS_TX_RX_ONLY);
    //UARTSetFifoMode(UART1, UART_INTERRUPT_ON_TX_NOT_FULL | UART_INTERRUPT_ON_RX_NOT_EMPTY);
    UARTSetLineControl(UART1, UART_DATA_SIZE_8_BITS | UART_PARITY_NONE | UART_STOP_BITS_1);
    UARTSetDataRate(UART1, SYS_FREQ, 115200);
    UARTEnable(UART1, UART_ENABLE_FLAGS(UART_PERIPHERAL | UART_RX | UART_TX));
    
}

void RS232_run(){

    int i;

    // Variables for Tx
    unsigned char uartTxBuffer[256];
    unsigned long numTxBytes = 0;
    unsigned long lastTxSent = 0;
    unsigned char TxStatus = 0;
    int numToWrite;



    // Variables for Rx
    unsigned char uartRxBuffer[256];
    unsigned long numRxBytes = 0;
    unsigned long lastRxRead = 0;
    unsigned char RxStatus = 0;
    int numToRead = 0;
    
    while(1){
        
        if((USBGetDeviceState() < CONFIGURED_STATE) ||
           (USBIsDeviceSuspended() == TRUE))
        {
            continue;   
        }


        if(!HIDRxHandleBusy(USBOutHandle))	
        {
            numToWrite = usbOutBuffer[0];

            memcpy(&uartTxBuffer[numTxBytes & 0xFF],&usbOutBuffer[1],numToWrite);
            numTxBytes += numToWrite;

            //Re-arm the OUT endpoint for the next packet
            USBOutHandle = HIDRxPacket(HID_EP,(BYTE*)&usbOutBuffer,64);
        }

        if(UARTReceivedDataIsAvailable(UART1))
        {
            uartRxBuffer[numRxBytes & 0xFF] = UARTGetDataByte(UART1);
            numRxBytes++;
        }
        

        if (UARTTransmitterIsReady(UART1))
        {
            if((numTxBytes - lastTxSent) > 127) TxStatus = 1;
            if((numTxBytes - lastTxSent) > 0){
                UARTSendDataByte(UART1, uartTxBuffer[lastTxSent & 0xFF]);
                lastTxSent++;
            }
        }

        if(!HIDTxHandleBusy(USBInHandle)){

            numToRead = numRxBytes-lastRxRead;

            if(numToRead > 63){
                numToRead = 63;
                RxStatus = 1;
            }

            if(numToRead > 0){
                // Clear the USB in buffer
                memset(usbInBuffer,0,64);

                // Fill the buffer, first byte is length and status
                usbInBuffer[0] = numToRead;// | (RxStatus << 7) | (TxStatus << 6);
                for(i=0;i<numToRead;i++){
                    usbInBuffer[i+1] = uartRxBuffer[(lastRxRead+i) & 0xFF];
                }

                // Send the USB in buffer packet, and update the last byte read
                USBInHandle = HIDTxPacket(HID_EP,(BYTE*)&usbInBuffer[0],sizeof(usbInBuffer));
                lastRxRead += numToRead;
                mLED = !mLED;
            }
        }


    }
}
